//========================================================================================
//
//	File:				FWArDyna.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	(c) 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFound.hpp"

#ifndef FWARDYNA_H
#include "FWArDyna.h"
#endif

#ifndef FWOBJREG_H
#include "FWObjReg.h"
#endif

#ifndef FWPRISTR_H
#include "FWPriStr.h"
#endif

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#pragma segment FWArchiv

//========================================================================================
//	Class FW_CDynamicArchiver
//========================================================================================

FW_SPrivDynamicArchiverGlobals FW_CDynamicArchiver::gGlobals;


//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::GetDynamicArchiverGlobals
//----------------------------------------------------------------------------------------

FW_SPrivDynamicArchiverGlobals& FW_CDynamicArchiver::GetDynamicArchiverGlobals()
{
//	FW_SPrivDynamicArchiverGlobals *globals = (FW_SPrivDynamicArchiverGlobals*)FW_CPrivTaskGlobals::GetTaskGlobals(kDynamicArchiverGlobalsOffset);
	FW_SPrivDynamicArchiverGlobals *globals = &gGlobals;

	if (globals->gMapClassNameToClassLabel == 0)
		Initialize(*globals);
	return *globals;
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::FW_CDynamicArchiver
//----------------------------------------------------------------------------------------

FW_CDynamicArchiver::FW_CDynamicArchiver(const char* classLabel,
							const char* className,
							FW_CPrivNameToLabelMap& localNameToLabelMap,
							FW_CPrivLabelToIOFunctionMap& localLabelToIOFunctionMap,
							FW_CReadableStream::InputFunction inputFunction,
							FW_CWritableStream::OutputFunction outputFunction)
{
	localNameToLabelMap.AddPair(new FW_SPrivArcStr(className), new FW_SPrivArcStr(classLabel));
	localLabelToIOFunctionMap.AddPair(new FW_SPrivArcStr(classLabel), new FW_SPrivArcFun(inputFunction, outputFunction));
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::FW_CDynamicArchiver
//----------------------------------------------------------------------------------------

FW_CDynamicArchiver::FW_CDynamicArchiver(const char* oldVersionClassLabel,
										 FW_CPrivLabelToIOFunctionMap& localLabelToIOFunctionMap,
										 FW_CReadableStream::InputFunction inputFunction)
{
	localLabelToIOFunctionMap.AddPair(new FW_SPrivArcStr(oldVersionClassLabel), new FW_SPrivArcFun(inputFunction, 0));
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::InputObject
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::InputObject(FW_CReadableStream& readableStream,
									  void*& object)
{
	FW_CReadableStream::InputFunction inputFunction = (FW_CReadableStream::InputFunction)readableStream.InputObject(ReadClassLabel);

	object = readableStream.InputObject(inputFunction);
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::ReadClassLabel
//
// Notes:
//	1.	You may have expected this method to return a pointer to the class label. Doing
//		so, however, would result in two problems. First, the pointer returned would have
//		to be to a buffer allocated by this method and who would delete it? Second, even
//		if a pointer to a class label was returned, by returning an:
//			'FW_CReadableStream::InputFunction'
//		pointer, we only have to map a particular class label once (via the map
//		'gMapClassLabelToObjectIOFunction') for the entire 'readableStream'. That is,
//		if there are "n" occurrences of a particular class label in a 'readableStream',
//		one reference to 'gMapClassLabelToObjectIOFunction' is made instead of "n".
//----------------------------------------------------------------------------------------

void* FW_CDynamicArchiver::ReadClassLabel(FW_CReadableStream& readableStream)
{
	// Read the class label of the object from the archive.
	char classLabel[kMaxClassLabelLength];
	short labelLength;
	readableStream >> labelLength;
	FW_ASSERT(labelLength < kMaxClassLabelLength);
	readableStream.Read(classLabel, labelLength);
	classLabel[labelLength] = 0;
	return LookupInputFunction(classLabel);
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::LookupOutputFunction
//----------------------------------------------------------------------------------------

FW_CWritableStream::OutputFunction 
FW_CDynamicArchiver::LookupOutputFunction(const char* classLabel)
{
	FW_SPrivArcStr tempLabel(classLabel);
	FW_ASSERT(GetLabelToIOFunctionMap().ContainsKey(&tempLabel));
	void *temp = GetLabelToIOFunctionMap().ValueAtKey(&tempLabel);
	FW_ASSERT(temp != 0);
	FW_SPrivArcFun *arcfun = (FW_SPrivArcFun*) temp;
	FW_CWritableStream::OutputFunction result = arcfun->fOutputFunction;
	FW_ASSERT(result != 0);
	return result;
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::LookupInputFunction
//----------------------------------------------------------------------------------------

FW_CReadableStream::InputFunction
FW_CDynamicArchiver::LookupInputFunction(const char* classLabel)
{
	FW_SPrivArcStr tempLabel(classLabel);
	FW_ASSERT(GetLabelToIOFunctionMap().ContainsKey(&tempLabel));
	void *temp = GetLabelToIOFunctionMap().ValueAtKey(&tempLabel);
	FW_ASSERT(temp != 0);
	FW_SPrivArcFun *arcfun = (FW_SPrivArcFun*) temp;
	FW_CReadableStream::InputFunction result = arcfun->fInputFunction;
	FW_ASSERT(result != 0);
	return result;
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::OutputObject
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::OutputObject(FW_CWritableStream& writableStream,
									   const void* object,
									   const char* className)
{
	FW_SPrivArcStr tempName(className);
	FW_ASSERT(GetNameToLabelMap().ContainsKey(&tempName));
	void *temp = GetNameToLabelMap().ValueAtKey(&tempName);
	FW_SPrivArcStr *label = (FW_SPrivArcStr *) temp;

	// Write the class label out to the archive.
	writableStream.OutputObject(WriteClassLabel, label->fString);

	// Write the 'object' data out to the archive.
	FW_CWritableStream::OutputFunction outputFunction = LookupOutputFunction(label->fString);
	writableStream.OutputObject(outputFunction, object);
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::WriteClassLabel
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::WriteClassLabel(FW_CWritableStream& writableStream,
										  const void* classLabel)
{
	short labelLength = FW_PrimitiveStringLength((const char *)classLabel);
	writableStream << labelLength;
	writableStream.Write(classLabel, labelLength);
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::Initialize
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::Initialize(FW_SPrivDynamicArchiverGlobals& globals)
{
	globals.gMapClassNameToClassLabel = new FW_CPrivNameToLabelMap();
	globals.gMapClassLabelToObjectIOFunction = new FW_CPrivLabelToIOFunctionMap();
}

//----------------------------------------------------------------------------------------
// FW_CDynamicArchiver::Terminate
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::Terminate()
{
	FW_SPrivDynamicArchiverGlobals& globals = GetDynamicArchiverGlobals();
	delete globals.gMapClassNameToClassLabel;
	delete globals.gMapClassLabelToObjectIOFunction;
}


//========================================================================================
//	Struct FW_SPrivArcStr
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_SPrivArcStr::FW_SPrivArcStr
//----------------------------------------------------------------------------------------

FW_SPrivArcStr::FW_SPrivArcStr(const FW_SPrivArcStr& string) :
	fString(string.fString)
{
	FW_ASSERT(this->fString != NULL);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcStr::FW_SPrivArcStr
//----------------------------------------------------------------------------------------

FW_SPrivArcStr::FW_SPrivArcStr(const char* string) :
	fString(string)
{
	FW_ASSERT(this->fString != NULL);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcStr::operator=
//----------------------------------------------------------------------------------------

FW_SPrivArcStr& FW_SPrivArcStr::operator=(const FW_SPrivArcStr& string)
{
	this->fString = string.fString;
	return *this;
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcStr::operator==
//----------------------------------------------------------------------------------------

FW_Boolean FW_SPrivArcStr::operator==(const FW_SPrivArcStr& string) const
{
	return FW_PrimitiveStringEqual(fString, string.fString);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcStr::operator!=
//----------------------------------------------------------------------------------------

FW_Boolean FW_SPrivArcStr::operator!=(const FW_SPrivArcStr& string) const
{
	return !operator==(string);
}

//========================================================================================
//	Struct FW_SPrivArcFun
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_SPrivArcFun::FW_SPrivArcFun
//----------------------------------------------------------------------------------------

FW_SPrivArcFun::FW_SPrivArcFun(const FW_SPrivArcFun& ioFunctions) :
	fInputFunction(ioFunctions.fInputFunction),
	fOutputFunction(ioFunctions.fOutputFunction)
{
	FW_ASSERT(fInputFunction != 0);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcFun::FW_SPrivArcFun
//----------------------------------------------------------------------------------------

FW_SPrivArcFun::FW_SPrivArcFun(FW_CReadableStream::InputFunction inputFunction,
							   FW_CWritableStream::OutputFunction outputFunction) :
	fInputFunction(inputFunction),
	fOutputFunction(outputFunction)
{
	FW_ASSERT(fInputFunction != 0);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcFun& FW_SPrivArcFun::operator=
//----------------------------------------------------------------------------------------

FW_SPrivArcFun& FW_SPrivArcFun::operator=(const FW_SPrivArcFun& ioFunctions)
{
	fInputFunction = ioFunctions.fInputFunction;
	fOutputFunction = ioFunctions.fOutputFunction;
	return *this;
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcFun::operator==
//----------------------------------------------------------------------------------------

FW_Boolean FW_SPrivArcFun::operator==(const FW_SPrivArcFun& ioFunctions) const
{
	return (fInputFunction == ioFunctions.fInputFunction) && (fOutputFunction == ioFunctions.fOutputFunction);
}

//----------------------------------------------------------------------------------------
// FW_SPrivArcFun::operator!=
//----------------------------------------------------------------------------------------

FW_Boolean FW_SPrivArcFun::operator!=(const FW_SPrivArcFun& ioFunctions) const
{
	return !operator==(ioFunctions);
}

//----------------------------------------------------------------------------------------
//	FW_CDynamicArchiver::MergeNameToLabelMaps
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::MergeNameToLabelMaps(
									FW_CPrivNameToLabelMap& local,
									FW_CPrivNameToLabelMap& global) 
{
	global.Merge(local);
}

//----------------------------------------------------------------------------------------
//	FW_CDynamicArchiver::MergeLabelToIOFunctionMaps
//----------------------------------------------------------------------------------------

void FW_CDynamicArchiver::MergeLabelToIOFunctionMaps(
									FW_CPrivLabelToIOFunctionMap& local,
									FW_CPrivLabelToIOFunctionMap& global) 
{
	global.Merge(local);
}

//----------------------------------------------------------------------------------------
//	FW_InitializeArchiving
//----------------------------------------------------------------------------------------

#ifdef FW_HAS_INSTANCE_DATA
void FW_ARCHIVE_FUNC_ATTR FW_InitializeArchiving(FW_Instance rootInstance)
{
FW_UNUSED(rootInstance);

	FW_MergeArchiverMaps();
}
#endif

#ifdef FW_BUILD_WIN16
#include "FWWinMod.h"

void FW_InitializeArchiving(FW_Instance rootInstance)
{
	FW_CWin16Module* partModule = GetModuleFromHInstance(rootInstance);
	FW_ASSERT(partModule != NULL);
	
	// Iterate through all the entries in the module reference table
	WORD refCount = partModule->GetNumModuleRefs();
	for(WORD i = 0; i < refCount; i++)
	{
		FW_CWin16Module* refModule = partModule->GetModuleRef(i);
		FW_ASSERT(refModule != NULL);

		HINSTANCE hInst = refModule->GetInstance();
		FARPROC lpMyProc = GetProcAddress(hInst, "_FW_MERGEARCHIVERMAPSHOOK");
		if (lpMyProc)
			(*lpMyProc) ();
	}
	
	// Finally, merge the archiver map for the root DLL instance
	FARPROC lpMyProc = GetProcAddress(rootInstance, "_FW_MERGEARCHIVERMAPSHOOK");
	if (lpMyProc)
		(*lpMyProc) ();
}
#endif

FW_CPrivNameToLabelMap::FW_CPrivNameToLabelMap()
{
}

FW_CPrivNameToLabelMap::~FW_CPrivNameToLabelMap()
{
}

FW_Boolean	FW_CPrivNameToLabelMap::KeysMatch(FW_PrivKeyType key1, FW_PrivKeyType key2) const
{
	FW_SPrivArcStr *k1 = (FW_SPrivArcStr *) key1;
	FW_SPrivArcStr *k2 = (FW_SPrivArcStr *) key2;
	return *k1 == *k2;
}

FW_CPrivLabelToIOFunctionMap::FW_CPrivLabelToIOFunctionMap()
{
}

FW_CPrivLabelToIOFunctionMap::~FW_CPrivLabelToIOFunctionMap()
{
}

FW_Boolean	FW_CPrivLabelToIOFunctionMap::KeysMatch(FW_PrivKeyType key1, FW_PrivKeyType key2) const
{
	FW_SPrivArcStr *k1 = (FW_SPrivArcStr *) key1;
	FW_SPrivArcStr *k2 = (FW_SPrivArcStr *) key2;
	return *k1 == *k2;
}
